# Notify Service PHP Client

## Installation

In your shell of choice, execute:

`composer install myriad/notify-client-php`

Publish the config file:

`php artisan vendor:publish --provider 'Myriad\NotifyClient\Providers\NotificationServiceProvider' --tag='config'`

## Configuration

Set the `NOTIFICATIONS_TOKEN` environmental variable and specify any channels and subscriptions you wish to support
in the config file.

## Usage

The main entrypoint for the SDK is the Client:

```php
$client = new Myriad\NotifyClient\Client();
```

To construct a notification, you may use a NotificationBuilder:

```php
$notification = NotificationBuilder::new()
    ->title('Message title')
    ->summary('Message summary')
    ->filter(FilterBuilder::new()->forSubscriptions('weather_warning');
```

The filter restricts the set of registrants that receive the message. `FilterBuilder` can be used to construct many common queries.

To send the notification:
```
$client->createNotification($notification);
```

The `FilterBuilder` may also be used to query notifications and notification groups:

```php
$client->queryNotifications(FilterBuilder::new()->toUids($array_of_uids));
```

If you need to write a fancier query than what the `FilterBuilder` can handle, you can use `FilterBuilder::raw` to construct queries using MongoDB's query syntax:

```php
$filter = FilterBuilder::raw([ 'scheduledAt' => [ '$gte' => Carbon::now()->toRfc3339String() ] ]);
$client->queryNotifications($filter);
```
