<?php namespace Myriad\NotifyClient\Builders;

use function debug_backtrace;
use const DEBUG_BACKTRACE_IGNORE_ARGS;
use Exception;

class FilterBuilder {

  private $channels = null;

  private $raw = null;

  private $serviceTypes = null;

  private $shouldValidate = true;

  private $subscriptions = null;

  private $uids = null;

  private $variants = null;

  public function onChannels($val) {
    $this->assertNotRaw();
    $channels = is_array($val) ? $val : func_get_args();
    $this->channels = $channels;
    return $this;
  }

  public function forSubscriptions($val) {
    $this->assertNotRaw();
    $subscriptions = is_array($val) ? $val : func_get_args();
    $this->subscriptions = $subscriptions;
    return $this;
  }

  public function toUids($val) {
    $this->assertNotRaw();
    $this->uids = is_array($val) ? $val : func_get_args();
    return $this;
  }

  public function viaServices($val) {
    $this->assertNotRaw();
    $this->serviceTypes = is_array($val) ? $val : func_get_args();
    return $this;
  }

  public function toVariants($val) {
    $this->assertNotRaw();
    $this->variants = is_array($val) ? $val : func_get_args();
    return $this;
  }

  public function withoutValidation() {
    $this->assertNotRaw();
    $this->shouldValidate = false;
    return $this;
  }

  public function build() {
    if ($this->isRaw()) {
      return $this->raw;
    }

    $filter = [];
    if ($this->channels) {
      $filter['channels'] = ['$in' => $this->channels];
    }
    if ($this->serviceTypes) {
      $filter['registrants.serviceType'] = ['$in' => $this->serviceTypes];
    }
    if ($this->subscriptions) {
      $filter['registrants.subscriptions'] = ['$in' => $this->subscriptions];
    }
    if ($this->uids) {
      $filter['uid'] = ['$in' => $this->uids];
    }
    if ($this->variants) {
      $filter['registrants.variant'] = ['$in' => $this->variants];
    }

    return $filter;
  }

  public function isRaw() {
    return !is_null($this->raw);
  }

  public static function new () {
    return new self();
  }

  public static function raw(array $filter) {
    $filterObj = new self();
    $filterObj->configureRaw($filter);
    return $filterObj;
  }

  private function configureRaw($filter) {
    $this->shouldValidate = false;
    $this->raw = $filter;
  }

  private function assertNotRaw() {
    if ($this->isRaw()) {
      $trace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
      $methodName = $trace[1]['function'];
      throw new Exception("Cannot use `$methodName` when the FilterBuilder has been created using `FilterBuilder::raw`");
    }
  }

  function __get($name) {
    return $this->{$name};
  }
}