<?php namespace Myriad\NotifyClient\Builders;

use Exception;
use Myriad\NotifyClient\Models\Notification;
use Myriad\NotifyClient\Utilities\FluentBuilder;

/**
 * Class NotificationBuilder
 * @method NotificationBuilder title(string $value)
 * @method NotificationBuilder summary(string $value)
 * @method NotificationBuilder body(string $value)
 * @method NotificationBuilder image(string $value)
 * @method NotificationBuilder emailTitle(string $value)
 * @method NotificationBuilder emailSummary(string $value)
 * @method NotificationBuilder emailBody(string $value)
 * @method NotificationBuilder emailImage(string $value)
 * @method NotificationBuilder emailFrom(string $value)
 * @method NotificationBuilder emailReplyTo(string $value)
 * @method NotificationBuilder emailBodyHtml(string $value)
 * @method NotificationBuilder emailHeaders(array $value)
 * @method NotificationBuilder pushTitle(string $value)
 * @method NotificationBuilder pushSummary(string $value)
 * @method NotificationBuilder pushBody(string $value)
 * @method NotificationBuilder pushImage(string $value)
 * @method NotificationBuilder pushSilent(boolean $value)
 * @method NotificationBuilder pushTag(string $value)
 * @method NotificationBuilder pushBadge(integer $value)
 * @method NotificationBuilder pushPriority(string $value)
 * @method NotificationBuilder pushData(array $value)
 * @method NotificationBuilder smsTitle(string $value)
 * @method NotificationBuilder smsSummary(string $value)
 * @method NotificationBuilder smsBody(string $value)
 * @method NotificationBuilder smsImage(string $value)
 * @method NotificationBuilder metadata(array $value)
 * @method NotificationBuilder showInHistory(boolean $value)
 * @method NotificationBuilder scheduledAt(\Carbon\Carbon $value)
 * @package Myriad\NotifyClient\Client
 */
class NotificationBuilder extends FluentBuilder {

  /**
   * @var Notification
   */
  protected $object;

  protected $filterBuilder = null;

  function __construct() {
    $this->object = new Notification();
  }

  public function filter(FilterBuilder $filterBuilder) {
    $this->filterBuilder = $filterBuilder;
    return $this;
  }

  public function build() {
    if (is_null($this->filterBuilder)) {
      throw new Exception('A filter was not provided to the NotificationBuilder.');
    }
    $this->object->filter = $this->filterBuilder->build();
    return $this->object;
  }

  function __get($name) {
    return $this->{$name};
  }
}