<?php namespace Myriad\NotifyClient\Client\Helpers;

use Exception;
use Myriad\NotifyClient\Builders\NotificationBuilder;
use Myriad\NotifyClient\Client\ValidatesAgainstConfig;
use Myriad\NotifyClient\Models\Notification;
use Myriad\NotifyClient\Utilities\BuiltWithConfig;
use Myriad\NotifyClient\Utilities\HydratesModels;

class NotificationHelper {

  use BuiltWithConfig;
  use HydratesModels;
  use ValidatesAgainstConfig;

  protected function getItemClass() {
    return Notification::class;
  }

  public function makeArray($object) {
    if ($object instanceof NotificationBuilder) {
      return $object->build()->toArray();
    }
    if ($object instanceof Notification) {
      return $object->toArray();
    }
    throw new Exception('Object must be a Notification, or NotificationBuilder');
  }

  public function maybeValidateBuilder ($notification) {
    if ($notification instanceof NotificationBuilder) {
      $filterBuilder = $notification->filterBuilder;
      if ($filterBuilder->shouldValidate) {
        $this->maybeAssertChannelsAreValid($filterBuilder->channels ?? []);
        $this->maybeAssertSubscriptionsAreValid($filterBuilder->subscriptions ?? []);
      }
    }
  }

  public function uri($notificationId = null) {
    return "notifications" . ($notificationId ? "/$notificationId" : '');
  }
}