<?php namespace Myriad\NotifyClient\Client;

use function func_get_args;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\BadResponseException;

/**
 * Class HttpWrapper
 * This class serves as a facade for interacting with the uri builder and the http client
 * @package Myriad\NotifyClient\Client
 */
class HttpWrapper {

  /**
   * Guzzle HTTP client.
   * @var Client
   */
  private $http;

  function __construct($http) {
    $this->http = $http;
  }

  /**
   * @param $verb
   * @param $arguments
   * @return mixed
   * @throws \Exception
   */
  private function makeRequest($verb, $arguments) {
    $arg1 = array_get($arguments, 0);
    $arg2 = array_get($arguments, 1);

    $url = '';
    if (is_string($arg1)) {
      $url = $arg1;
      $json = $arg2;
    } else {
      $json = $arg1;
    }

    try {
      $response = call_user_func_array([$this->http, $verb], [$url, ['json' => $json]]);

      if ($response->getStatusCode() === 204) {
        // is this what we want to do?
        return null;
      }

      $responseJson = json_decode($response->getBody(), true);
      return array_get($responseJson, 'data');
    } catch (BadResponseException $e) {
      $baseUrl = $this->http->getConfig('base_uri');
      $response = $e->getResponse();
      throw new \Exception(
        "Client error response [url] $baseUrl$url [status code] {$response->getStatusCode()} [body] {$response->getBody()}"
      );
    }
  }

  public function get () {
    return $this->makeRequest('get', func_get_args());
  }

  public function put () {
    return $this->makeRequest('put', func_get_args());
  }

  public function post () {
    return $this->makeRequest('post', func_get_args());
  }

  public function delete () {
    return $this->makeRequest('delete', func_get_args());
  }
}