<?php namespace Myriad\NotifyClient\Models;

use function array_set;
use Myriad\NotifyClient\Utilities\BasicModel;

/**
 * Class Notification
 * @property string $id
 * @property string $title
 * @property string $summary
 * @property string $body
 * @property string $image
 * @property string $emailTitle
 * @property string $emailSummary
 * @property string $emailBody
 * @property string $emailImage
 * @property string $emailFrom
 * @property string $emailReplyTo
 * @property string $emailBodyHtml
 * @property array $emailHeaders
 * @property string $pushTitle
 * @property string $pushSummary
 * @property string $pushBody
 * @property string $pushImage
 * @property boolean $pushSilent
 * @property string $pushTag
 * @property integer $pushBadge
 * @property string $pushPriority
 * @property array $pushData
 * @property string $smsTitle
 * @property string $smsSummary
 * @property string $smsBody
 * @property string $smsImage
 * @property array $filter
 * @property array $metadata
 * @property boolean $showInHistory
 * @property integer $failedCount
 * @property integer $canceledCount
 * @property integer $sentCount
 * @property \Carbon\Carbon $scheduledAt
 * @property \Carbon\Carbon $startedAt
 * @property \Carbon\Carbon $canceledAt
 * @package Myriad\NotifyClient\Models
 */
class Notification extends BasicModel {

  protected $dates = ['scheduledAt', 'startedAt', 'canceledAt'];

  // Getters

  public function getIdAttribute() {
    return $this->safeGetAttribute('_id');
  }

  public function getTitleAttribute() {
    return $this->safeGetAttribute('content.title');
  }

  public function getSummaryAttribute() {
    return $this->safeGetAttribute('content.summary');
  }

  public function getBodyAttribute() {
    return $this->safeGetAttribute('content.body');
  }

  public function getImageAttribute() {
    return $this->safeGetAttribute('content.image');
  }

  public function getEmailTitleAttribute() {
    return $this->safeGetAttribute('content.email.title');
  }

  public function getEmailSummaryAttribute() {
    return $this->safeGetAttribute('content.email.summary');
  }

  public function getEmailBodyAttribute() {
    return $this->safeGetAttribute('content.email.body');
  }

  public function getEmailImageAttribute() {
    return $this->safeGetAttribute('content.email.image');
  }

  public function getEmailFromAttribute() {
    return $this->safeGetAttribute('content.email.from');
  }

  public function getEmailReplyToAttribute() {
    return $this->safeGetAttribute('content.email.replyTo');
  }

  public function getEmailBodyHtmlAttribute() {
    return $this->safeGetAttribute('content.email.bodyHtml');
  }

  public function getEmailHeadersAttribute() {
    return $this->safeGetAttribute('content.email.headers');
  }

  public function getPushTitleAttribute() {
    return $this->safeGetAttribute('content.push.title');
  }

  public function getPushSummaryAttribute() {
    return $this->safeGetAttribute('content.push.summary');
  }

  public function getPushBodyAttribute() {
    return $this->safeGetAttribute('content.push.body');
  }

  public function getPushImageAttribute() {
    return $this->safeGetAttribute('content.push.image');
  }

  public function getPushSilentAttribute() {
    return $this->safeGetAttribute('content.push.silent');
  }

  public function getPushTagAttribute() {
    return $this->safeGetAttribute('content.push.tag');
  }

  public function getPushBadgeAttribute() {
    return $this->safeGetAttribute('content.push.badge');
  }

  public function getPushPriorityAttribute() {
    return $this->safeGetAttribute('content.push.priority');
  }

  public function getPushDataAttribute() {
    return $this->safeGetAttribute('content.push.data');
  }

  public function getSmsTitleAttribute() {
    return $this->safeGetAttribute('content.sms.title');
  }

  public function getSmsSummaryAttribute() {
    return $this->safeGetAttribute('content.sms.summary');
  }

  public function getSmsBodyAttribute() {
    return $this->safeGetAttribute('content.sms.body');
  }

  public function getSmsImageAttribute() {
    return $this->safeGetAttribute('content.sms.image');
  }

  // Setters

  public function setTitleAttribute($title) {
    $this->safeSetAttribute('content.title', $title);
  }

  public function setSummaryAttribute($summary) {
    $this->safeSetAttribute('content.summary', $summary);
  }

  public function setBodyAttribute($body) {
    $this->safeSetAttribute('content.body', $body);
  }

  public function setImageAttribute($body) {
    $this->safeSetAttribute('content.image', $body);
  }

  public function setEmailTitleAttribute($value) {
    $this->safeSetAttribute('content.email.title', $value);
  }

  public function setEmailSummaryAttribute($value) {
    $this->safeSetAttribute('content.email.summary', $value);
  }

  public function setEmailBodyAttribute($value) {
    $this->safeSetAttribute('content.email.body', $value);
  }

  public function setEmailImageAttribute($value) {
    $this->safeSetAttribute('content.email.image', $value);
  }

  public function setEmailFromAttribute($value) {
    $this->safeSetAttribute('content.email.from', $value);
  }

  public function setEmailReplyToAttribute($value) {
    $this->safeSetAttribute('content.email.replyTo', $value);
  }

  public function setEmailBodyHtmlAttribute($value) {
    $this->safeSetAttribute('content.email.bodyHtml', $value);
  }

  public function setEmailHeadersAttribute($value) {
    $this->safeSetAttribute('content.email.headers', $value);
  }

  public function setPushTitleAttribute($value) {
    $this->safeSetAttribute('content.push.title', $value);
  }

  public function setPushSummaryAttribute($value) {
    $this->safeSetAttribute('content.push.summary', $value);
  }

  public function setPushBodyAttribute($value) {
    $this->safeSetAttribute('content.push.body', $value);
  }

  public function setPushImageAttribute($value) {
    $this->safeSetAttribute('content.push.image', $value);
  }

  public function setPushSilentAttribute($value) {
    $this->safeSetAttribute('content.push.silent', $value);
  }

  public function setPushTagAttribute($value) {
    $this->safeSetAttribute('content.push.tag', $value);
  }

  public function setPushBadgeAttribute($value) {
    $this->safeSetAttribute('content.push.badge', $value);
  }

  public function setPushPriorityAttribute($value) {
    $this->safeSetAttribute('content.push.priority', $value);
  }

  public function setPushDataAttribute($value) {
    $this->safeSetAttribute('content.push.data', $value);
  }

  public function setSmsTitleAttribute($value) {
    $this->safeSetAttribute('content.sms.title', $value);
  }

  public function setSmsSummaryAttribute($value) {
    $this->safeSetAttribute('content.sms.summary', $value);
  }

  public function setSmsBodyAttribute($value) {
    $this->safeSetAttribute('content.sms.body', $value);
  }

  public function setSmsImageAttribute($value) {
    $this->safeSetAttribute('content.sms.image', $value);
  }

  // Utils

  private function safeGetAttribute($key, $default = null) {
    return array_get($this->attributes, $key, $default);
  }

  private function safeSetAttribute($key, $value) {
    array_set($this->attributes, $key, $value);
  }

}